@@ -9,20 +9,8 @@
 #ifndef WLR_RENDER_INTERFACE_H
 #define WLR_RENDER_INTERFACE_H
 
-#ifndef MESA_EGL_NO_X11_HEADERS
-#define MESA_EGL_NO_X11_HEADERS
-#endif
-#ifndef EGL_NO_X11
-#define EGL_NO_X11
-#endif
-#ifndef EGL_NO_PLATFORM_SPECIFIC_TYPES
-#define EGL_NO_PLATFORM_SPECIFIC_TYPES
-#endif
-
-#include <EGL/egl.h>
-#include <EGL/eglext.h>
 #include <stdbool.h>
-#include <wayland-server-protocol.h>
+#include <wayland-server-core.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/render/wlr_texture.h>
 #include <wlr/types/wlr_box.h>
@@ -30,6 +18,8 @@
 #include <wlr/render/dmabuf.h>
 
 struct wlr_renderer_impl {
+	bool (*bind_buffer)(struct wlr_renderer *renderer,
+		struct wlr_buffer *buffer);
 	void (*begin)(struct wlr_renderer *renderer, uint32_t width,
 		uint32_t height);
 	void (*end)(struct wlr_renderer *renderer);
@@ -42,24 +32,24 @@ struct wlr_renderer_impl {
 		const float color[static 4], const float matrix[static 9]);
 	void (*render_ellipse_with_matrix)(struct wlr_renderer *renderer,
 		const float color[static 4], const float matrix[static 9]);
-	const enum wl_shm_format *(*formats)(
-		struct wlr_renderer *renderer, size_t *len);
-	bool (*format_supported)(struct wlr_renderer *renderer,
-		enum wl_shm_format fmt);
+	const uint32_t *(*get_shm_texture_formats)(struct wlr_renderer *renderer,
+		size_t *len);
 	bool (*resource_is_wl_drm_buffer)(struct wlr_renderer *renderer,
 		struct wl_resource *resource);
 	void (*wl_drm_buffer_get_size)(struct wlr_renderer *renderer,
 		struct wl_resource *buffer, int *width, int *height);
-	const struct wlr_drm_format_set *(*get_dmabuf_formats)(
+	const struct wlr_drm_format_set *(*get_dmabuf_texture_formats)(
 		struct wlr_renderer *renderer);
-	enum wl_shm_format (*preferred_read_format)(struct wlr_renderer *renderer);
-	bool (*read_pixels)(struct wlr_renderer *renderer, enum wl_shm_format fmt,
+	const struct wlr_drm_format_set *(*get_dmabuf_render_formats)(
+		struct wlr_renderer *renderer);
+	uint32_t (*preferred_read_format)(struct wlr_renderer *renderer);
+	bool (*read_pixels)(struct wlr_renderer *renderer, uint32_t fmt,
 		uint32_t *flags, uint32_t stride, uint32_t width, uint32_t height,
 		uint32_t src_x, uint32_t src_y, uint32_t dst_x, uint32_t dst_y,
 		void *data);
 	struct wlr_texture *(*texture_from_pixels)(struct wlr_renderer *renderer,
-		enum wl_shm_format fmt, uint32_t stride, uint32_t width,
-		uint32_t height, const void *data);
+		uint32_t fmt, uint32_t stride, uint32_t width, uint32_t height,
+		const void *data);
 	struct wlr_texture *(*texture_from_wl_drm)(struct wlr_renderer *renderer,
 		struct wl_resource *data);
 	struct wlr_texture *(*texture_from_dmabuf)(struct wlr_renderer *renderer,
@@ -70,6 +60,7 @@ struct wlr_renderer_impl {
 	bool (*blit_dmabuf)(struct wlr_renderer *renderer,
 		struct wlr_dmabuf_attributes *dst,
 		struct wlr_dmabuf_attributes *src);
+	int (*get_drm_fd)(struct wlr_renderer *renderer);
 };
 
 void wlr_renderer_init(struct wlr_renderer *renderer,
